@@ -21,7 +21,13 @@ const NetworksManager = new Lang.Class({
 
         let uri = 'resource:///org/gnome/Polari/data/networks.json';
         let file = Gio.File.new_for_uri(uri);
-        file.load_contents_async(null, Lang.bind(this, this._onContentsReady));
+        let success, data;
+        try {
+            [success, data, ] = file.load_contents(null);
+            this._parseNetworks(data);
+        } catch(e) {
+            log('Failed to load network list: ' + e.message);
+        }
     },
 
     _onContentsReady: function(f, res) {
@@ -32,21 +38,26 @@ const NetworksManager = new Lang.Class({
             log('Failed to load network list: ' + e.message);
             return;
         }
+        if (this._parseNetworks(data))
+            this.emit('changed');
+    },
 
+    _parseNetworks: function(data) {
         let networks;
         try {
             networks = JSON.parse(data);
         } catch(e) {
             log('Failed to parse network list: ' + e.message);
-            return;
+            return false;
         }
 
+        this._networksById.clear();
         this._networks = networks;
         this._networks.forEach(Lang.bind(this,
             function(network) {
                 this._networksById.set(network.id, network);
             }));
-        this.emit('changed');
+        return true;
     },
 
     _lookupNetwork: function(id) {
